use std::io::Result;
use std::str::from_utf8;
use nom::{IResult, newline};
use package::{Package, Entry};

named!(header<Entry>, do_parse!(
    entry: alt!(
        tag!("%FILENAME%")     => {|_| Entry::Filename }
      | tag!("%NAME%")         => {|_| Entry::Name }
      | tag!("%BASE%")         => {|_| Entry::Base }
      | tag!("%VERSION%")      => {|_| Entry::Version }
      | tag!("%DESC%")         => {|_| Entry::Description }
      | tag!("%GROUPS%")       => {|_| Entry::Groups }
      | tag!("%CSIZE%")        => {|_| Entry::PackageSize }
      | tag!("%ISIZE%")        => {|_| Entry::InstallSize }
      | tag!("%SHA256SUM%")    => {|_| Entry::SHA256Sum }
      | tag!("%PGPSIG%")       => {|_| Entry::PGPSig }
      | tag!("%URL%")          => {|_| Entry::Url }
      | tag!("%LICENSE%")      => {|_| Entry::License }
      | tag!("%ARCH%")         => {|_| Entry::Arch }
      | tag!("%BUILDDATE%")    => {|_| Entry::BuildDate }
      | tag!("%PACKAGER%")     => {|_| Entry::Packager }
      | tag!("%REPLACES%")     => {|_| Entry::Replaces }
      | tag!("%DEPENDS%")      => {|_| Entry::Depends }
      | tag!("%CONFLICTS%")    => {|_| Entry::Conflicts }
      | tag!("%PROVIDES%")     => {|_| Entry::Provides }
      | tag!("%OPTDEPENDS%")   => {|_| Entry::OptDepends }
      | tag!("%MAKEDEPENDS%")  => {|_| Entry::MakeDepends }
      | tag!("%CHECKDEPENDS%") => {|_| Entry::CheckDepends }
      | tag!("%FILES%")        => {|_| Entry::Files }
      | tag!("%DELTAS%")       => {|_| Entry::Deltas }
    ) >>
    newline >>
    (entry)
));

named!(values<Vec<&str>>, many0!(
    map_opt!(
        take_until_and_consume!("\n"),
        |s| from_utf8(s).ok().and_then(|s| match s {
            "" => None,
            s => Some(s)
        })
    )
));

named!(desc<Vec<(Entry, Vec<&str>)>>, many0!(
    do_parse!(
        header: header >>
        values: values >>
        opt!(tag!("\n")) >>
        (header, values)
    )
));

impl Package {
    pub fn read_desc(&mut self, data: &[u8]) -> Result<()> {
        match desc(data) {
            IResult::Done(_, ref o) => {
                for &(ref entry, ref values) in o {
                    match *entry {
                        Entry::Name => assert_eq!(self.name, values[0]),
                        Entry::Version => assert_eq!(self.version, values[0]),
                        Entry::Arch => {
                            self.arch = values[0].into();
                        }
                        ref entry => {
                            self.metadata.insert(entry.clone(), (entry, values).into());
                        }
                    };
                }
            }
            error => println!("error: {:?}", error),
        }
        Ok(())
    }
}

#[test]
fn test_cool() {}
