use std::str;
use std::collections::HashMap;
use nom::{IResult, space, alpha, alphanumeric, digit, multispace, not_line_ending};
use package::Package;

named!(comment <&[u8],&str>,
    chain!(
        tag!("#") ~
        comment_txt: map_res!(not_line_ending, str::from_utf8),
        || { comment_txt }
    )
);

named!(key_value <&[u8],(&str,&str)>,
    do_parse!(
        key: map_res!(alphanumeric, str::from_utf8) >>
            opt!(space) >>
            tag!("=") >>
            opt!(space) >>
        val: map_res!(
            take_until_either!("\n;"),
            str::from_utf8
        ) >>
        opt!(space) >>
        opt!(do_parse!(
            tag!(";") >>
            not_line_ending >>
            ()
        )) >>
        opt!(multispace) >>
        (key, val)
    )
);

named!(keys_and_values_aggregator<&[u8], Vec<(&str,&str)> >, many0!(key_value));

fn keys_and_values(input: &[u8]) -> IResult<&[u8], HashMap<&str, &str>> {
    let mut h: HashMap<&str, &str> = HashMap::new();

    match keys_and_values_aggregator(input) {
        IResult::Done(i, tuple_vec) => {
            for &(k, v) in &tuple_vec {
                h.insert(k, v);
            }
            IResult::Done(i, h)
        }
        IResult::Incomplete(a) => IResult::Incomplete(a),
        IResult::Error(a) => IResult::Error(a),
    }
}

named!(name_parser<&str>,
    chain!(
        tag!("hello") ~
        space? ~
        name: map_res!(
            alpha,
            str::from_utf8
        ) ,

        || name
    )
);

#[test]
fn test_name_parser() {
    let pkginfo = b"# Generated by makepkg 5.0.1
# using fakeroot version 1.21
# Sun Oct 30 16:09:47 UTC 2016
pkgname = repose-git
pkgver = 6.2.10.gbab93f3-1
pkgdesc = A archlinux repo building tool
url = http://github.com/vodik/repose
builddate = 1477843787
packager = Simon Gomizelj <simongmzlj@gmail.com>
size = 63488
arch = x86_64
license = GPL
conflict = repose
provides = repose
depend = pacman
depend = libarchive
depend = gnupg
makedepend = git
makedepend = ragel
";

    let res = key_value(pkginfo);
    println!("{:?}", res);
}

#[test]
fn test_pkginfo_with_backup() {
    let pkginfo = b"pkgname = example
backup = etc/example/conf
";

    let res = key_value(pkginfo);
    println!("{:?}", res);
}

#[test]
fn test_invalid_pkginfo_entry() {
    let pkginfo = b"pkgname = invalid_pkginfo_entry
badentry = etc/example/conf
";

    let res = key_value(pkginfo);
    println!("{:?}", res);
}

#[test]
fn test_empty_pkginfo_entry() {
    let pkginfo = b"pkgname = empty_pkginfo_entry
url =
";

    let res = key_value(pkginfo);
    println!("{:?}", res);
}

#[test]
fn test_makepkgopt() {
    let pkginfo = b"pkgname = ttf-ms-win10-sea
makepkgopt = strip
makepkgopt = !debug
";

    let res = key_value(pkginfo);
    println!("{:?}", res);
}
