use libc;
use std::io;
use std::fs::File;
use std::os::unix::io::*;
use std::ffi::CString;

pub struct Storage {
    fd: RawFd,
}

impl Storage {
    pub fn new(root: &str) -> io::Result<Storage> {
        let path = CString::new(root)?;

        let fd = unsafe { libc::open(path.as_ptr(), libc::O_RDONLY | libc::O_DIRECTORY) };
        match fd {
            -1 => Err(io::Error::last_os_error()),
            _ => Ok(Storage { fd: fd }),
        }
    }

    pub fn open(&self, file: &str) -> io::Result<File> {
        let path = CString::new(file)?;

        let fd = unsafe {
            libc::openat(self.fd,
                         path.as_ptr() as *const libc::c_char,
                         libc::O_RDONLY)
        };

        match fd {
            -1 => Err(io::Error::last_os_error()),
            _ => unsafe { Ok(File::from_raw_fd(fd)) },
        }
    }
}

impl Drop for Storage {
    fn drop(&mut self) {
        unsafe {
            libc::close(self.fd);
        }
    }
}
